#version 330

uniform sampler2D depthTexture;
uniform sampler2D frameTexture;

uniform float velocityScale;
uniform mat4 inverseViewProjMat;
uniform mat4 prevViewProj;

uniform int isFast;

noperspective in vec2 fUV;

out vec4 fragColor;

void main() {
	float zOverW=texture(depthTexture, fUV).r;  
	vec4 H=vec4(fUV.x*2-1, fUV.y*2-1, zOverW, 1);   
	vec4 D=inverseViewProjMat*H;
	vec4 worldPos=D/D.w;

	vec4 currentPos=H;
	vec4 previousPos=prevViewProj*worldPos;
	previousPos/=previousPos.w;

	vec2 blurVec=((currentPos-previousPos)/2.0).xy;

	vec4 final = texture(frameTexture, fUV);
	if (blurVec.x == 0 && blurVec.y == 0) {
		fragColor=final;
	} else {
		blurVec *= velocityScale;

		if(isFast == 0) {
			int samples=100;
			for (int i = 1; i < samples; ++i) {
				vec2 offset = blurVec * (float(i)/float(samples-1)-0.5);
				final += texture(frameTexture, fUV + offset);
			}
			fragColor=final/float(samples);

		} else {
			int samples=50;
			vec2 tfUV = fUV;  
			for(int i = 1; i < samples; ++i) { 
			  tfUV += blurVec * (float(i)/float(samples-1)-0.3);
			  final += texture(frameTexture, tfUV); 
			}  
			fragColor = final/float(samples);
		}
	}
}