#version 330

struct Light {
	vec3 position;
	vec3 intensity;
	float cAttenuation;
	float lAttenuation;
	float qAttenuation;

	int isSpotlight;
	float cosOuterCone;
	float cosInnerCone;
	vec3 coneDirection;
};
	
const float ambientCoefficient = 0.2f;
const int numberOfLights=14;
Light lights[numberOfLights];
uniform Light light0;
uniform Light light1;
uniform Light light2;
uniform Light light3;
uniform Light light4;
uniform Light light5;
uniform Light light6;
uniform Light light7;
uniform Light light8;
uniform Light light9;
uniform Light light10;
uniform Light light11;
uniform Light light12;
uniform Light light13;

uniform mat4 model;
uniform vec3 cameraPosition;
uniform sampler2D colorTexture;
uniform float shininess;
uniform vec3 specularColor;

in vec2 fragUv;
in vec3 fragPosition;
in vec3 fragNormal;

layout(location = 0) out vec4 fragColor;

void main() {
	lights[0] = light0;
	lights[1] = light1;
	lights[2] = light2;
	lights[3] = light3;
	lights[4] = light4;
	lights[5] = light5;
	lights[6] = light6;
	lights[7] = light7;
	lights[8] = light8;
	lights[9] = light9;
	lights[10] = light10;
	lights[11] = light11;
	lights[12] = light12;
	lights[13] = light13;

	vec3 N=normalize(transpose(inverse(mat3(model)))*fragNormal);
	vec3 surfacePosition=vec3(model*vec4(fragPosition, 1));
	vec4 texColor=texture(colorTexture, fragUv);
	vec3 vecToCamera=normalize(cameraPosition-surfacePosition);

	vec3 color = ambientCoefficient * texColor.rgb;
	for (int index = 0; index < numberOfLights; index++) {
		vec3 L = lights[index].position-surfacePosition;		
		float d = length(L);
		L = normalize(L);
		float intensity=max(0.0, dot(L, N));
		
		//attenuation
		float attenuation=1.0/(1.0 + (lights[index].lAttenuation * d) + (lights[index].qAttenuation * d * d));

		//spotlight
		if (lights[index].isSpotlight == 1) {
			float spotDot = dot(-L, normalize(lights[index].coneDirection));
			attenuation *= smoothstep(lights[index].cosOuterCone, lights[index].cosInnerCone, spotDot);			
		}

		if (attenuation != 0.0) {
			//diffuse
			float diffuseCoefficient=max(0.0, dot(N, L));
			vec3 diffuseLight = diffuseCoefficient * texColor.rgb * lights[index].intensity;
			
			//specular
			float specularCoefficient=0.0;
			if(diffuseCoefficient > 0.0  && shininess != 0.0) {				
				specularCoefficient = pow(max(0.0, dot(vecToCamera, reflect(-L, N))), shininess);
			}
			if(specularCoefficient>0.6)
				specularCoefficient=1.0;
			else if(specularCoefficient>0.3)
				specularCoefficient=0.2;
			else
				specularCoefficient=0.0;
				
			vec3 specularLight=specularCoefficient*specularColor*lights[index].intensity;
						
			float diffToon;
			if (intensity > 0.85)
				diffToon= 1.0;
			else if (intensity > 0.6)
				diffToon= 0.6;
			else if (intensity > 0.3)
				diffToon= 0.3;
			else if (intensity > 0.1)
				diffToon= 0.1;
			
			//combined
			color += ((diffuseLight*diffToon) + specularLight) * attenuation;
		}
	}
	fragColor = vec4(pow(color, vec3(1.0/2.2)), texColor.a);
}